#encoding: UTF-8 
=begin
/***************************************************************************
 *   ©2016-2017 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end


require 'optparse'
require 'optparse/time'
require 'ostruct'
require_relative 'logging'
require_relative 'version'
# require_relative 'translating'
require_relative 'constants'

class ArgParser 
  # Class level logger. This is a static class. 
  self.extend(Logging)
  # self.extend(Translating)
  @@log = init_logger()

  # Returns a structure describing the options.
  #
  def self.parse(args)
    if args.empty?
      puts usage 
      exit true
    end
    # The options specified on the command line will be collected in
    # <b>options</b>.  No defaults. Most options are optional and do not
    # have to be set at all.
    # The others must be named for each transformation or be set in the
    # configuration-file.
    options = OpenStruct.new
    options.target = nil

    op = OptionParser.new do |opts|
      opts.banner = usage

      opts.on("-d", "--debug", 'Be verbose') do  
        $log_level = Logger::DEBUG
        @@log.level = $log_level
      end

      opts.on("-sOURCE", "--source=SOURCE", 'Source document (html)') do |so|
        options.source = so
      end

      opts.on("-oUT", "--out=GLOSSAR", 'Glossar-file (html)') do |ta|
        options.target = ta
      end

      opts.on("-tEMPLATE", "--template=TEMPLATE", 'Template (html)') do |tpl|
        options.template = tpl
      end

      opts.on("-cONFIG", "--config=CONFIG", 'Configuration-file') do |cfg|
        options.config = cfg
      end

      opts.on("-h", "--help", 'Show this message') do  
        puts opts
        exit true
      end

      opts.on("-v", "--version", 'Show program version') do  
        puts APPNAME.dup << ", version " << VERSION
        exit true
      end
    end
    begin
      op.parse!(args)
    rescue OptionParser::ParseError => er
      msg = "ERROR! Unsuitable or incomplete program-arguments" << ": %s" %er.message	
      puts msg
      puts "Start this program with parameter -h or --help to see the usage-message."
      exit false
    end
    @@log.debug('options are ' << options.to_s)

    options
  end  # parse()

=begin
        Shows the usage-message
=end
  def self::usage
    msg = "\n\tUsage: html2index -s input.html [-o output.html] [-c config-file] [-t template.html] [-d]"
    msg << "\n\n\t* Will print to stdout, if the output-file is not provided."
    msg << "\n\t* Adapt ~/.config/HTML2Index/config to your needs.\n\n"
  end


end  

